'~This is file libFBla.bas, a FB-library for linear algebra
'~Version 0.2
'~(C) 2011 by Thomas[ dot ]Freiherr[ at ]gmx{ dot }net
'~License LGPLv 2
'
'~Create new variables of type vector or matrix and use them
'~like real numbers (and mind the rules of linear algebra).
'
'~Three new types are provided
'~1) LA_S = scalar (user defined precision, lib based on this type)
'~2) LA_V = vector of scalars (variable number of rows)
'~3) LA_M = matrix of scalars (variable number of rows and columns)
'~Note: index is zero based (allways)
'
'~All new types can be used at module level as well as inside
'~other SUBs/FUNCTIONs/UDTs.
'
'~Adjust the library

'~error handling
#DEFINE LA_CheckIndex '~Check if index is inside the given range
'#DEFINE LA_ErrorsNone '~no error checking and no output
'#DEFINE LA_StopOnError
'#DEFINE LA_StopOnWarning

'~String to separate matrix/vector values on output
#IFNDEF LA_TR
#DEFINE LA_TR !"\t"
#ENDIF

'~Expression to format the output, ie
'#DEFINE LA_FORMAT(_V_) FORMAT(_V_, "###.##")
#IFNDEF LA_FORMAT
#DEFINE LA_FORMAT(_V_) STR(_V_)
#ENDIF

'~String for new line
#IFNDEF NL
#DEFINE NL !"\n"
#ENDIF

'~The variable type for the library
#IFNDEF LA_S
 TYPE AS DOUBLE LA_S
#ENDIF

'~The limit for singular determinants
#IFNDEF LA_Eps
 #DEFINE LA_Eps (1e-15)
#ENDIF

'~Code starts here
'~*
'~*  Some macros (internal use only)
'~************************************************************

'~Print an error message, stop if required
#MACRO LA_Err(_T_)
 #IFNDEF LA_ErrorsNone
  ?:?"LA-error(" & __LINE__ & ") "; _T_; "!"
  #IFDEF LA_StopOnError
   END 1
  #ENDIF
 #ENDIF
#ENDMACRO

'~Print a warning, stop if required
#MACRO LA_Warn(_T_)
 #IFNDEF LA_ErrorsNone
  ?:?"LA-warning(" & __LINE__ & ") "; _T_; "!"
  #IFDEF LA_StopOnWarning
   END 1
  #ENDIF
 #ENDIF
#ENDMACRO

'~Pointer to a local scalar
#DEFINE LA_L      CAST(LA_S PTR, SADD(    Dat))
'~Pointer to a global scalar
#DEFINE LA_G(_G_) CAST(LA_S PTR, SADD(_G_.Dat))
'~Separators for STRING input
#DEFINE LA_SEPS ANY !";\n\l\t\v"

'~*
'~*  The vector TYPE
'~************************************************************

TYPE LA_V
  DECLARE CONSTRUCTOR(BYREF V AS STRING, BYVAL Mo AS INTEGER = 0)
  DECLARE CONSTRUCTOR(BYVAL N AS UINTEGER = 1, BYVAL S AS LA_S = 0.0)
  DECLARE PROPERTY Len_() AS LA_S
  DECLARE PROPERTY Val_(BYVAL N AS UINTEGER) AS LA_S
  DECLARE PROPERTY Val_(BYVAL N AS UINTEGER, BYVAL V AS LA_S)
  DECLARE FUNCTION Ubound_() AS UINTEGER
  DECLARE SUB Clear_(BYVAL N AS INTEGER = 0, BYVAL S AS LA_S = .0)
  DECLARE SUB ReDim_(BYVAL N AS UINTEGER = 1, BYVAL S AS LA_S = .0)
  DECLARE OPERATOR CAST () AS STRING
  DECLARE OPERATOR += (BYVAL S AS LA_S)
  DECLARE OPERATOR -= (BYVAL S AS LA_S)
  DECLARE OPERATOR *= (BYVAL S AS LA_S)
  DECLARE OPERATOR /= (BYVAL S AS LA_S)
  DECLARE OPERATOR \= (BYVAL S AS LA_S)
  DECLARE OPERATOR ^= (BYVAL S AS LA_S)
'~these member operators act element-wise!!!
  DECLARE OPERATOR += (BYREF V AS LA_V)
  DECLARE OPERATOR -= (BYREF V AS LA_V)
  DECLARE OPERATOR *= (BYREF V AS LA_V)
  DECLARE OPERATOR /= (BYREF V AS LA_V)
  DECLARE OPERATOR \= (BYREF V AS LA_V)
  DECLARE OPERATOR ^= (BYREF V AS LA_V)
'~These variables are meant for internal use only
  AS STRING Dat
  AS UINTEGER Mx
END TYPE

'~Create and initialize a new vector
'~ V =  "1,2,5,9" (plain text = slow) or
'~Mo <> 0 means binary data (like 'MKD(1) & MKD(2) & MKD(5) & MKD(9)')
CONSTRUCTOR LA_V(BYREF V AS STRING, BYVAL Mo AS INTEGER = 0)
  IF Mo THEN
    Mx = LEN(V) \ SIZEOF(LA_S)
    IF Mx * SIZEOF(LA_S) = LEN(V) THEN Dat = V : EXIT CONSTRUCTOR
    LA_Err("string lenght missmatch: CONSTRUCTOR LA_V")
  ELSE
    VAR a = 1
    DO
      VAR e = INSTR(a, V, LA_SEPS) : IF e < a THEN EXIT DO
      Dat &= STRING(SIZEOF(LA_S), 0) : LA_L[Mx] = VAL(MID(V, a, e - a))
      Mx += 1 : a = e + 1
    LOOP
    Dat &= STRING(SIZEOF(LA_S), 0) : LA_L[Mx] = VAL(MID(V, a))
    Mx += 1
  END IF
END CONSTRUCTOR

'~Create an new vector with N elements (defaults to 0.0)
'~Use optional S parameter to specify initial value
CONSTRUCTOR LA_V(BYVAL N AS UINTEGER = 1, BYVAL S AS LA_S = 0.0)
  Mx = N
  Dat = STRING(N * SIZEOF(LA_S), 0)
  IF S THEN FOR i AS INTEGER = 0 TO Mx - 1 : LA_L[i] = S : NEXT
END CONSTRUCTOR

'~Return the length of the vector
PROPERTY LA_V.Len_() AS LA_S
  VAR x = LA_L[0] * LA_L[0]
  FOR i AS INTEGER = 1 TO Mx - 1
    x += LA_L[i] * LA_L[i]
  NEXT : RETURN x ^ 0.5
END PROPERTY

'~Return the value at index N
PROPERTY LA_V.Val_(BYVAL N AS UINTEGER) AS LA_S
#IFDEF LA_CheckIndex
  IF N < Mx THEN RETURN LA_L[N]
  LA_Warn("index out of range: V.Val_(" & N & ")")
  RETURN .0
#ELSE
  RETURN LA_L[N]
#ENDIF
END PROPERTY

'~Set a new value V at index N
PROPERTY LA_V.Val_(BYVAL N AS UINTEGER, BYVAL V AS LA_S)
#IFDEF LA_CheckIndex
  IF N <= LEN(Dat) / SIZEOF(LA_S) THEN
#ENDIF
    LA_L[N] = V
#IFDEF LA_CheckIndex
  ELSE
    LA_Warn("index out of range: V.Val_(" & N & ") =")
  END IF
#ENDIF
END PROPERTY

'~Clear_() --> set all values = zero
'~Clear_(3) --> set values 0 TO 2 = zero
'~Clear_(-3) --> set values Ubound_ TO Ubound_-2 = zero
'~Clear_(2, 3.14) --> set values 0 TO 1 = 3.14
'~Clear_( , -0.707) --> set all values = -0.707
SUB LA_V.Clear_(BYVAL N AS INTEGER = 0, BYVAL S AS LA_S = .0)
  VAR a = 0, e = Mx - 1, st = 1
  SELECT CASE N
  CASE IS > 0
    e = IIF(N <= e, N - 1, e)
  CASE IS < 0
    a = e
    e = IIF(e > -N, a + N + 1, 0)
    st = -1
  END SELECT
  FOR i AS INTEGER = a TO e STEP st
    LA_L[i] = S
  NEXT
END SUB

'~Shrink or grow a vector (S = init value)
SUB LA_V.ReDim_(BYVAL N AS UINTEGER = 1, BYVAL S AS LA_S = .0)
  IF N <> Mx THEN Mx = IIF(N, N, 1) ELSE EXIT SUB
  VAR la = LEN(Dat), ln = Mx * SIZEOF(LA_S)
  IF ln < la THEN Dat = LEFT(Dat, ln) : EXIT SUB
  Dat &= STRING(ln - la, 0) : IF S = .0 THEN EXIT SUB
  FOR i AS INTEGER = la \ SIZEOF(LA_S) - 1 TO Mx - 1
    LA_L[i] = S
  NEXT
END SUB

'~Return max index
FUNCTION LA_V.Ubound_() AS UINTEGER
  RETURN Mx - 1
END FUNCTION

OPERATOR LA_V.CAST () AS STRING
  VAR t = LA_FORMAT(LA_L[0])
  FOR i AS INTEGER = 1 TO Mx -1
    t &= LA_TR & LA_FORMAT(LA_L[i])
  NEXT : RETURN t
END OPERATOR

'~computes a member operation vector -> scalar (internal)
#MACRO LA_CALCM_VS(_OP_)
  FOR i AS INTEGER = 0 TO Mx - 1
    LA_L[i] _OP_ S
  NEXT
#ENDMACRO

'~Add a scalar
OPERATOR LA_V.+=(BYVAL S AS LA_S)
  LA_CALCM_VS(+=)
END OPERATOR

'~Deduct a scalar
OPERATOR LA_V.-=(BYVAL S AS LA_S)
  LA_CALCM_VS(-=)
END OPERATOR

'~Multiply by a scalar
OPERATOR LA_V.*=(BYVAL S AS LA_S)
  LA_CALCM_VS(*=)
END OPERATOR

'~Divide by a scalar
OPERATOR LA_V./=(BYVAL S AS LA_S)
  IF S THEN
    LA_CALCM_VS(/=)
  ELSE
    LA_Err("division by zero: V(.) /= 0.0")
  END IF
END OPERATOR

'~Integer divide by a scalar
OPERATOR LA_V.\=(BYVAL S AS LA_S)
  IF S THEN
    LA_CALCM_VS(\=)
  ELSE
    LA_Err("division by zero: V(.) \= 0.0")
  END IF
END OPERATOR

'~Exponent by a scalar
OPERATOR LA_V.^=(BYVAL S AS LA_S)
  LA_CALCM_VS(^=)
END OPERATOR


'~Get elements# of two vectors, calc min (n) (internal)
#MACRO LA_CHECK_VV(V1, V2, _FN_, _OP_)
  VAR l1 = V1.Ubound_(), l2 = V2.Ubound_()
  IF l1 <> l2 THEN
    LA_Warn("dimensions mismatch: " & _FN_ & _
                       "V(" & l1 + 1 & ")" & _OP_ & _
                       "V(" & l2 + 1 & ")")
  END IF
  VAR n = IIF(l1 > l2, l1, l2)
#ENDMACRO

'~computes a member operation vector -> vector element-wise (internal)
#MACRO LA_CALCM_VV(_OP_)
  LA_CHECK_VV(THIS, V, "", " " & #_OP_ & " ")
  FOR i AS INTEGER = 0 TO n
    LA_L[i] _OP_ V.Val_(i)
  NEXT
#ENDMACRO

'~Add a vector element-wise
OPERATOR LA_V.+=(BYREF V AS LA_V)
  LA_CALCM_VV(+=)
END OPERATOR

'~Deduct a vector element-wise
OPERATOR LA_V.-=(BYREF V AS LA_V)
  LA_CALCM_VV(-=)
END OPERATOR

'~Multiply a vector element-wise
OPERATOR LA_V.*=(BYREF V AS LA_V)
  LA_CALCM_VV(*=)
END OPERATOR

'~Divide a vector element-wise
OPERATOR LA_V./=(BYREF V AS LA_V)
  LA_CHECK_VV(THIS, V, "", " /= ")
  FOR i AS INTEGER = 0 TO n
    IF V.Val_(i) THEN
      LA_L[i] /= V.Val_(i)
    ELSE
      LA_Err("division by zero element: V(.) /= V(" & i & ")")
    END IF
  NEXT
END OPERATOR

'~Integer divide a vector element-wise
OPERATOR LA_V.\=(BYREF V AS LA_V)
  LA_CHECK_VV(THIS, V, "", " \= ")
  FOR i AS INTEGER = 0 TO n
    IF V.Val_(i) THEN
      LA_L[i] \= V.Val_(i)
    ELSE
      LA_Err("integer division by zero element: V(.) \= V(" & i & ")")
    END IF
  NEXT
END OPERATOR

'~Exponent a vector element-wise
OPERATOR LA_V.^=(BYREF V AS LA_V)
  LA_CALCM_VV(^=)
END OPERATOR

'~*
'~*  Some global vector operators
'~************************************************************

'~compare vectors element-wise, up to the smaller Ubound_ (internal)
#MACRO LA_COMP_VV(_OP_, _T_)
  LA_CHECK_VV(V1, V2, "", " " & #_T_ & " ")
  FOR i AS INTEGER = 0 TO n
    IF V1.Val_(i) _OP_ V2.Val_(i) THEN RETURN 0
  NEXT : RETURN -1
#ENDMACRO

'~true if all elements of V1 are equal to the coresponding element of V2
OPERATOR =(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS INTEGER
  LA_COMP_VV(<>,=)
END OPERATOR

'~true if all elements of V1 are unequal to the coresponding element of V2
OPERATOR <>(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS INTEGER
  LA_COMP_VV(=,<>)
END OPERATOR

'~true if all elements of V1 smaller than the coresponding element of V2
OPERATOR <(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS INTEGER
  LA_COMP_VV(>=,<)
END OPERATOR

'~true if all elements of V1 greater than the coresponding element of V2
OPERATOR >(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS INTEGER
  LA_COMP_VV(<=,>)
END OPERATOR

'~true if all elements of V1 smaller than or equal to the coresponding element of V2
OPERATOR <=(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS INTEGER
  LA_COMP_VV(>,<=)
END OPERATOR

'~true if all elements of V1 greater than or equal to the coresponding element of V2
OPERATOR >=(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS INTEGER
  LA_COMP_VV(<,>=)
END OPERATOR


'~compare vector element-wise with scalar (internal)
#MACRO LA_COMP_VS(_OP_)
  FOR i AS INTEGER = 0 TO V.Ubound_()
    IF V.Val_(i) _OP_ S THEN RETURN 0
  NEXT : RETURN -1
#ENDMACRO

'~true if all elements of the Vector are equal to the Scalar
OPERATOR =(BYREF V AS LA_V, BYREF S AS LA_S) AS INTEGER
  LA_COMP_VS(<>)
END OPERATOR

'~true if all elements of the Vector are unequal to the Scalar
OPERATOR <>(BYREF V AS LA_V, BYREF S AS LA_S) AS INTEGER
  LA_COMP_VS(=)
END OPERATOR

'~true if all elements of the Vector are smaller than the Scalar
OPERATOR <(BYREF V AS LA_V, BYREF S AS LA_S) AS INTEGER
  LA_COMP_VS(>=)
END OPERATOR

'~true if all elements of the Vector are greater than the Scalar
OPERATOR >(BYREF V AS LA_V, BYREF S AS LA_S) AS INTEGER
  LA_COMP_VS(<=)
END OPERATOR

'~true if all elements of the Vector are smaller than or equal to the Scalar
OPERATOR <=(BYREF V AS LA_V, BYREF S AS LA_S) AS INTEGER
  LA_COMP_VS(>)
END OPERATOR

'~true if all elements of the Vector are greater than or equal to the Scalar
OPERATOR >=(BYREF V AS LA_V, BYREF S AS LA_S) AS INTEGER
  LA_COMP_VS(<)
END OPERATOR


'~true if the Scalar is equal to all elements of the Vector
OPERATOR =(BYREF S AS LA_S, BYREF V AS LA_V) AS INTEGER
  RETURN V = S
END OPERATOR

'~true if the Scalar is unequal to all elements of the Vector
OPERATOR <>(BYREF S AS LA_S, BYREF V AS LA_V) AS INTEGER
  RETURN V <> S
END OPERATOR

'~true if the Scalar is smaller than all elements of the Vector
OPERATOR <(BYREF S AS LA_S, BYREF V AS LA_V) AS INTEGER
  RETURN V > S
END OPERATOR

'~true if the Scalar is greater than all elements of the Vector
OPERATOR >(BYREF S AS LA_S, BYREF V AS LA_V) AS INTEGER
  RETURN V < S
END OPERATOR

'~true if the Scalar is smaller than or equal to all elements of the Vector
OPERATOR <=(BYREF S AS LA_S, BYREF V AS LA_V) AS INTEGER
  RETURN V >= S
END OPERATOR

'~true if the Scalar is greater than or equal to all elements of the Vector
OPERATOR >=(BYREF S AS LA_S, BYREF V AS LA_V) AS INTEGER
  RETURN V <= S
END OPERATOR


'~Vector = -Vector
OPERATOR -(BYREF V AS LA_V) AS LA_V
  VAR x = V
  FOR i AS INTEGER = 0 TO V.Ubound_()
    x.Val_(i) = -V.Val_(i)
  NEXT : RETURN x
END OPERATOR


'~get Ubound_ (n) and prepare a vector result (x) (internal)
#MACRO LA_FOR_VS_V()
  VAR n = V.Ubound_(), x = LA_V(n + 1)
  FOR i AS INTEGER = 0 TO n
#ENDMACRO

'~Vector = Vector * Scalar
OPERATOR *(BYREF V AS LA_V, BYVAL S AS LA_S) AS LA_V
  LA_FOR_VS_V()
    x.Val_(i) = V.Val_(i) * S
  NEXT : RETURN x
END OPERATOR

'~Vector = Scalar * Vector
OPERATOR *(BYREF S AS LA_S, BYREF V AS LA_V) AS LA_V
  RETURN V * S
END OPERATOR

'~Vector = Vector / Scalar
OPERATOR /(BYREF V AS LA_V, BYVAL S AS LA_S) AS LA_V
  IF S THEN RETURN V * 1 / S
  LA_Err("division by zero: V(.) / 0.0")
  RETURN V
END OPERATOR

'~Vector = Scalar / Vector
OPERATOR /(BYREF S AS LA_S, BYREF V AS LA_V) AS LA_V
  LA_FOR_VS_V()
    VAR f = x.Val_(i)
    IF f THEN
      x.Val_(i) = S / f
    ELSE
      LA_Err("division by zero: S / V(" & i & ")")
    END IF
  NEXT : RETURN x
END OPERATOR

'~Vector = Vector + Scalar
OPERATOR +(BYREF V AS LA_V, BYVAL S AS LA_S) AS LA_V
  VAR x = V : x += S : RETURN x
END OPERATOR

'~Vector = Scalar + Vector
OPERATOR +(BYREF S AS LA_S, BYREF V AS LA_V) AS LA_V
  VAR x = V : x += S : RETURN x
END OPERATOR

'~Vector = Vector - Scalar
OPERATOR -(BYREF V AS LA_V, BYVAL S AS LA_S) AS LA_V
  VAR x = V : x -= S : RETURN x
END OPERATOR

'~Vector = Scalar - Vector
OPERATOR -(BYREF S AS LA_S, BYREF V AS LA_V) AS LA_V
  VAR x = (-V) : x += S : RETURN x
END OPERATOR


'~Check size (2 vectors), run FOR with scalar result (x) (internal)
#MACRO LA_FOR_VV_S(_FN_, _OP_)
  LA_CHECK_VV(V1, V2, _FN_, _OP_)
  DIM AS LA_S x
  FOR i AS INTEGER = 0 TO n
#ENDMACRO

'~Skalar = Vector * Vector (dot product)
OPERATOR *(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS LA_S
  LA_FOR_VV_S("", " dot ")
    x += V1.Val_(i) * V2.Val_(i)
  NEXT : RETURN x
END OPERATOR

'~Check size (2 vectors), run FOR with vector result (x) (internal)
#MACRO LA_FOR_VV_V(_FN_, _OP_)
  LA_CHECK_VV(V1, V2, _FN_, _OP_)
  VAR x = LA_V(n + 1)
  FOR i AS INTEGER = 0 TO n
#ENDMACRO

'~Vector = Vector + Vector
OPERATOR +(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS LA_V
  LA_FOR_VV_V("", " + ")
    x.Val_(i) = V1.Val_(i) + V2.Val_(i)
  NEXT : RETURN x
END OPERATOR

'~Vector = Vector - Vector
OPERATOR -(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS LA_V
  LA_FOR_VV_V("", " - ")
    x.VAL_(i) = V1.Val_(i) - V2.Val_(i)
  NEXT : RETURN x
END OPERATOR


'~*
'~*  Some vector SUBs / FUNCTIONs
'~************************************************************

'~normalize Vector to length L (default = 1.0)
FUNCTION Norm(BYREF V AS LA_V, BYVAL L AS LA_S = 1.0) AS LA_V
  VAR x = V.Len_()
  IF x THEN RETURN (L / x) * V
  LA_Warn("LA_V.Len_() = zero: Norm(V, S)")
  RETURN V
END FUNCTION

'~cross product of two Vectors
FUNCTION Cross(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS LA_V
  LA_FOR_VV_V("Cross ", ", ")
    VAR h1 = IIF( i > 0,  i - 1, n)
    VAR h2 = IIF(h1 > 0, h1 - 1, n)
    VAR j1 = IIF( i < n,  i + 1, 0)
    VAR j2 = IIF(j1 < n, j1 + 1, 0)
    x.VAL_(i) = V1.Val_(j1) * V2.Val_(j2) - V1.Val_(h1) * V2.Val_(h2)
  NEXT : RETURN x
END FUNCTION

'~Distance between two points V1 and V2
FUNCTION Dist(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS LA_S
  LA_FOR_VV_S("Dist ", ", ")
    VAR s = V1.Val_(i) - V2.Val_(i)
    x += s * s
  NEXT : RETURN x ^ 0.5
END FUNCTION

'~Ankle (rad) between two vectors V1 and V2
FUNCTION Ankle(BYREF V1 AS LA_V, BYREF V2 AS LA_V) AS LA_S
  VAR x = V1.Len_() * V2.Len_()
  IF x THEN RETURN ACOS((V1 * V2) / x)
  LA_Warn("zero vector[s]: Ankle(V1, V2)")
  RETURN .0 '!!! infinity
END FUNCTION


'~*
'~*  The matrix TYPE
'~************************************************************

TYPE LA_M
  DECLARE CONSTRUCTOR(BYREF V AS STRING)
  DECLARE CONSTRUCTOR(BYVAL R AS UINTEGER = 1, BYVAL C AS UINTEGER = 0, _
                      BYVAL V AS LA_S = .0, BYVAL Mo AS LA_S = .0)
  DECLARE SUB ReDim_(BYVAL R AS UINTEGER = 1, BYVAL C AS UINTEGER = 0, _
                     BYVAL V AS LA_S = .0, BYVAL D AS LA_S = .0)
  DECLARE FUNCTION Row_() AS UINTEGER
  DECLARE FUNCTION Col_() AS UINTEGER
  DECLARE FUNCTION Val_(BYVAL R AS UINTEGER, BYVAL C AS UINTEGER) AS LA_S
  '~ DECLARE PROPERTY Vec_(BYVAL R AS UINTEGER) AS LA_V
  '~ DECLARE PROPERTY Vec_(BYVAL C AS UINTEGER, BYREF V AS LA_V)
  DECLARE SUB Set_(BYVAL R AS UINTEGER, BYVAL C AS UINTEGER, _
                   BYVAL V AS LA_S = .0)
  DECLARE FUNCTION Det_() AS LA_S
  DECLARE SUB SwapR(BYVAL R1 AS UINTEGER, BYVAL R2 AS UINTEGER)
  DECLARE SUB SwapC(BYVAL C1 AS UINTEGER, BYVAL C2 AS UINTEGER)
  DECLARE OPERATOR CAST () AS STRING
  DECLARE OPERATOR += (BYVAL S AS LA_S)
  DECLARE OPERATOR -= (BYVAL S AS LA_S)
  DECLARE OPERATOR *= (BYVAL S AS LA_S)
  DECLARE OPERATOR /= (BYVAL S AS LA_S)
  DECLARE OPERATOR \= (BYVAL S AS LA_S)
  DECLARE OPERATOR ^= (BYVAL S AS LA_S)
'~these member operators act element-wise!!!
  DECLARE OPERATOR += (BYREF M AS LA_M)
  DECLARE OPERATOR -= (BYREF M AS LA_M)
  DECLARE OPERATOR *= (BYREF M AS LA_M)
  DECLARE OPERATOR /= (BYREF M AS LA_M)
  DECLARE OPERATOR \= (BYREF M AS LA_M)
  DECLARE OPERATOR ^= (BYREF M AS LA_M)
'~Private:
  AS STRING Dat
  AS UINTEGER Ro, Co, Mx
END TYPE

'~New matrix:
'~V = initial values as plain text (row by row)
'~    (ie | 11  12  13  14 |
'~        | 21  22  23  24 |
'~        | 31  32  33  34 |
'~      = !"11,12,13,14 \n 21,22,23,24 \n 31,32,33,34")
CONSTRUCTOR LA_M(BYREF V AS STRING)
  VAR c = 1 : Ro = 1
  FOR i AS INTEGER = 0 TO LEN(V) - 1
    SELECT CASE AS CONST V[i]
    'CASE ASC(",") : c += 1
    CASE ASC(";") : c += 1
    CASE ASC(NL) : Co = IIF(Co < c, c, Co) : c = 1 : Ro += 1
    END SELECT
  NEXT : Co = IIF(Co < c, c, Co)
  Mx = Ro * Co
  Dat = STRING(Mx * SIZEOF(LA_S), 0)
  VAR a = 1, pr = 0, p = 0
  DO
    VAR e = INSTR(a, V, LA_SEPS) : IF e < a THEN EXIT DO
    LA_L[p] = VAL(MID(V, a, e - a))
    SELECT CASE AS CONST V[e - 1]
    CASE ASC(" "), ASC(";"), ASC(!"\t"), ASC(!"\v") : p += Ro
    CASE ASC(NL), ASC(!"\r") : pr += 1 : p = pr
    END SELECT
    a = e + 1
  LOOP : LA_L[p] = VAL(MID(V, a))
END CONSTRUCTOR

'~New matrix:
'~R = number of rows (ie 2 = two rows, index 0 TO 1)
'~C = number of coloumns (as R, 0 or nothing means C = R)
'~V = initial value for plain matrix
'~D = initial value for diagonal (if R = C)
CONSTRUCTOR LA_M(BYVAL R AS UINTEGER = 1, BYVAL C AS UINTEGER = 0, _
                 BYVAL V AS LA_S = .0, BYVAL D AS LA_S = .0)
  Ro = IIF(R > 1, R, 1)
  Co = IIF(C = 0, Ro, C)
  Mx = Ro * Co
  Dat = STRING(Mx * SIZEOF(LA_S), 0)
  IF V THEN '~                                            set all values
    FOR i AS INTEGER = 0 TO Mx - 1
      LA_L[i] = V
    NEXT
  END IF
  IF D ANDALSO Ro = Co THEN '~                       set diagonal values
    FOR n AS INTEGER = 0 TO Ro - 1
      LA_L[n * Ro + n] = D
    NEXT
  END IF
END CONSTRUCTOR

'~ReDim matrix:
'~R = number of rows (ie 2 = two rows, index 0 TO 1)
'~C = number od coloumns (as R, 0 or nothing means C = R)
'~In case of groth
'~V = initial value for plain matrix
'~D = initial value for diagonal (if R = C)
SUB LA_M.ReDim_(BYVAL R AS UINTEGER = 1, BYVAL C AS UINTEGER = 0, _
                BYVAL V AS LA_S = .0, BYVAL D AS LA_S = .0)
  VAR rn = IIF(R > 1, R, 1)
  VAR cn = IIF(C = 0, rn, C)
  VAR old = Dat, Vold = CAST(LA_S PTR, SADD(old))
  Mx = rn * cn
  Dat = STRING(Mx * SIZEOF(LA_S), 0)

  VAR r1 = IIF(rn > Ro, Ro, rn), r2 = IIF(rn > Ro, rn, 0)
  VAR c1 = IIF(cn > Co, Co, cn), c2 = IIF(cn > Co, cn, 0)
  FOR c AS INTEGER = 0 TO c1 - 1 '~                      copy old values
    FOR r AS INTEGER = 0 TO r1 - 1
      LA_L[c * rn + r] = Vold[c * Ro + r]
    NEXT
  NEXT
  FOR c AS INTEGER = c1 TO c2 - 1 '~                new top right values
    FOR r AS INTEGER = 0 TO r1 - 1
      LA_L[c * rn + r] = V
    NEXT
  NEXT
  FOR c AS INTEGER = 0 TO c1 - 1 '~               new bottom left values
    FOR r AS INTEGER = r1 TO r2 - 1
      LA_L[c * rn + r] = V
    NEXT
  NEXT
  VAR dd = IIF(rn = cn, D, V) '~                 new bottom right values
  FOR c AS INTEGER = c1 TO c2 - 1
    FOR r AS INTEGER = r1 TO r2 - 1
      LA_L[c * rn + r] = IIF(r = c, dd, V)
    NEXT
  NEXT
  Ro = rn
  Co = cn
END SUB

'~Returns number of rows-1
FUNCTION LA_M.Row_() AS UINTEGER
  RETURN Ro - 1
END FUNCTION

'~Returns number of columns-1
FUNCTION LA_M.Col_() AS UINTEGER
  RETURN Co - 1
END FUNCTION

'~returns a coloumn (Spaltenvektor)
'~ PROPERTY LA_M.Vec_(BYVAL C AS UINTEGER) AS LA_V
  '~ IF C <= Co THEN RETURN Val[C]
  '~ LA_Err("R-index to big (LA_M.Vec_)")
'~ END PROPERTY

'~sets a coloumn (Spaltenvektor)
'~ PROPERTY LA_M.Vec_(BYVAL C AS UINTEGER, BYREF V AS LA_V)
  '~ IF C <= Co THEN Val[C] = V : EXIT PROPERTY
  '~ LA_Err("R-index to big (LA_M.Vec_ =)")
'~ END PROPERTY

'~Get value at Row and Col
FUNCTION LA_M.Val_(BYVAL R AS UINTEGER, BYVAL C AS UINTEGER) AS LA_S
#IFDEF LA_CheckIndex
  IF R >= Ro THEN
    LA_Warn("R-index#" & R & " out of range: M(" & Ro & ", .).Set_")
  ELSEIF C >= Co THEN
    LA_Warn("C-index#" & C & " out of range: M(., " & Co & ").Set_")
  ELSE
#ENDIF
    RETURN LA_L[C * Ro + R]
#IFDEF LA_CheckIndex
  END IF : RETURN .0
#ENDIF
END FUNCTION

'~Set a new Value at Row and Col
SUB LA_M.Set_(BYVAL R AS UINTEGER, BYVAL C AS UINTEGER, _
              BYVAL V AS LA_S = .0)
#IFDEF LA_CheckIndex
  IF R >= Ro THEN
    LA_Warn("R-index#" & R & " out of range: M(" & Ro & ", .).Set_")
  ELSEIF C >= Co THEN
    LA_Warn("C-index#" & C & " out of range: M(., " & Co & ").Set_")
  ELSE
#ENDIF
    LA_L[C * Ro + R] = V
#IFDEF LA_CheckIndex
  END IF
#ENDIF
END SUB

'~Compute the determinat (returns sub-determinant if Ro < Co)
FUNCTION LA_M.Det_() AS LA_S
  DIM AS LA_S det = 0, x1, x2
  VAR s1 = 1 + Ro, s2 = 1 - Ro, mm = Mx
  IF Co < Ro THEN
    LA_Err("matrix inquadratic: M.Det_(" & Ro & ", " & Co & ")")
    RETURN .0
  ELSEIF Co > Ro THEN
    LA_Warn("columns ignored: M.Det_(" & Ro & ", " & Co & ")")
    mm = Ro * Ro
  END IF
  FOR r AS INTEGER = 0 TO Ro - 1
    VAR p1 = r, p2 = r
    x1 = LA_L[p1]
    x2 = LA_L[p2]
    FOR i AS INTEGER = 2 TO Ro
      p1 += s1 : IF p1 >= mm THEN p1 -= mm
      x1 *= LA_L[p1]
      p2 += s2 : IF p2 < 0 THEN p2 += mm
      x2 *= LA_L[p2]
    NEXT
    det += x1 - x2
  NEXT : RETURN det
END FUNCTION

'~Swap two rows R1 and R2
SUB LA_M.SwapR(BYVAL R1 AS UINTEGER, BYVAL R2 AS UINTEGER)
  IF R1 < Ro ANDALSO R2 < Ro THEN
    VAR p = 0
    WHILE p < Mx
      SWAP LA_L[p + R1], LA_L[p + R2]
      p += Ro
    WEND
  ELSE
    LA_Warn("R-index to big: M(" & Ro & ", .).SwapR(" & R1 & ", " & R2 &")")
  END IF
END SUB

'~Swap two columns C1 and C2
SUB LA_M.SwapC(BYVAL C1 AS UINTEGER, BYVAL C2 AS UINTEGER)
  IF C1 < Co ANDALSO C2 < Co THEN
    VAR l = SIZEOF(LA_S) * Ro
    VAR p1 = C1 * l + 1
    VAR p2 = C2 * l + 1
    VAR d2 = MID(Dat, p2, l)
    MID(Dat, p2, l) = MID(Dat, p1, l)
    MID(Dat, p1, l) = d2
  ELSE
    LA_Warn("C-index to big: M(., " & Co & ").SwapC(" & C1 & ", " & C2 &")")
  END IF
END SUB

'~For output (printing)
OPERATOR LA_M.CAST() AS STRING
  VAR p = 0, t = LA_FORMAT(LA_L[p])
  FOR i AS INTEGER = 1 TO Mx - 1
    p += Ro
    IF p >= Mx THEN
      p += 1 - Mx
      t &= NL & LA_FORMAT(LA_L[p])
    ELSE
      t &= LA_TR & LA_FORMAT(LA_L[p])
    END IF
  NEXT : RETURN t
END OPERATOR

'~computes a member operation matrix -> scalar (internal)
#MACRO LA_CALCM_MS(_OP_)
  FOR i AS INTEGER = 0 TO Mx - 1
    LA_L[i] _OP_ S
  NEXT
#ENDMACRO

'~Add all values by Scalar
OPERATOR LA_M.+= (BYVAL S AS LA_S)
  LA_CALCM_MS(+=)
END OPERATOR

'~Subract all values by Scalar
OPERATOR LA_M.-= (BYVAL S AS LA_S)
  LA_CALCM_MS(-=)
END OPERATOR

'~Multiply all values by Scalar
OPERATOR LA_M.*= (BYVAL S AS LA_S)
  LA_CALCM_MS(*=)
END OPERATOR

'~Divide all values by Scalar
OPERATOR LA_M./= (BYVAL S AS LA_S)
  IF S THEN
    LA_CALCM_MS(/=)
    EXIT OPERATOR
  END IF
  LA_Err("division by zero: M(.,.) /= 0.0")
END OPERATOR

'~Integer divide all values by Scalar
OPERATOR LA_M.\= (BYVAL S AS LA_S)
  IF S THEN
    LA_CALCM_MS(\=)
    EXIT OPERATOR
  END IF
  LA_Err("division by zero: M(.,.) \= 0.0")
END OPERATOR

'~Exponent all values by Scalar
OPERATOR LA_M.^= (BYVAL S AS LA_S)
  LA_CALCM_MS(^=)
END OPERATOR


'~Check the dimension of two matrixes (internal)
#MACRO LA_CHECK_MM(_M1_, _M2_, _FN_, _OP_)
  IF _M1_.Ro <> _M2_.Ro THEN
    LA_Warn("row missmatch: " & _FN_ & _
                   "M(" & _M1_.Ro & ", .)" & _OP_ & _
                   "M(" & _M2_.Ro & ", .)")
  END IF
  IF _M1_.Co <> _M2_.Co THEN
    LA_Warn("column missmatch: " & _FN_ & _
                   "M(. , " & _M1_.Co & ")" & _OP_ & _
                   "M(. , " & _M2_.Co & ")")
  END IF
#ENDMACRO

'~runs FOR with index for matrix 1 and 2 (internal)
#MACRO LA_FOR_MM(_M1_, _M2_)
  FOR c AS INTEGER = 0 TO IIF(_M2_.Co > _M1_.Co, _M1_.Co, _M2_.Co) - 1
    FOR r AS INTEGER = 0 TO IIF(_M2_.Ro > _M1_.Ro, _M1_.Ro, _M2_.Ro) - 1
      VAR i1 = c * _M1_.Ro + r
      VAR i2 = c * _M2_.Ro + r
#ENDMACRO

'~ends FOR (internal)
#MACRO LA_NEXT_MM()
    NEXT
  NEXT
#ENDMACRO

'~compute member operation matrix element-wise (internal)
#MACRO LA_CALCM_MM(_OP_)
  LA_CHECK_MM(THIS, M, "", " " & #_OP_ & " ")
  LA_FOR_MM(THIS, M)
    LA_L[i1] _OP_ LA_G(M)[i2]
  LA_NEXT_MM()
#ENDMACRO

'~Add a matrix element-wise
OPERATOR LA_M.+= (BYREF M AS LA_M)
  LA_CALCM_MM(+=)
END OPERATOR

'~Subtract a matrix element-wise
OPERATOR LA_M.-= (BYREF M AS LA_M)
  LA_CALCM_MM(-=)
END OPERATOR

'~Subtract a matrix element-wise
OPERATOR LA_M.*= (BYREF M AS LA_M)
  LA_CALCM_MM(*=)
END OPERATOR

'~Devide a matrix element-wise
OPERATOR LA_M./= (BYREF M AS LA_M)
  LA_CHECK_MM(THIS, M, "", " /= ")
  LA_FOR_MM(THIS, M)
    IF LA_G(M)[i2] THEN
      LA_L[i1] /= LA_G(M)[i2]
    ELSE
      LA_Err("division by zero: M(.,.) /= M(" & i2 MOD M.Ro & ", " _
                                              & i2 \ M.Ro & ")")
    END IF
  LA_NEXT_MM()
END OPERATOR

'~Integer devide a matrix element-wise
OPERATOR LA_M.\= (BYREF M AS LA_M)
  LA_CHECK_MM(THIS, M, "", " \= ")
  LA_FOR_MM(THIS, M)
    IF LA_G(M)[i2] THEN
      LA_L[i1] \= LA_G(M)[i2]
    ELSE
      LA_Err("integer division by zero: M(.,.) \= M(" & i2 MOD M.Ro & ", " _
                                                      & i2 \ M.Ro & ")")
    END IF
  LA_NEXT_MM()
END OPERATOR

'~Exponent a matrix element-wise
OPERATOR LA_M.^= (BYREF M AS LA_M)
  LA_CALCM_MM(^=)
END OPERATOR


'~*
'~*  Matrix functions
'~************************************************************

'~Creates a transposed matrix
FUNCTION Transp OVERLOAD(BYREF M AS LA_M) AS LA_M
  VAR x = M : x.Ro = M.Co : x.Co = M.Ro : IF x.Ro = 1 THEN RETURN x
  VAR p = M.Ro
  FOR i AS INTEGER = 1 TO x.Mx - 2
    LA_G(x)[i] = LA_G(M)[p]
    p += M.Ro : IF p >= M.Mx THEN p += 1 - M.Mx
  NEXT : RETURN x
END FUNCTION

'~Creates a matrix as transposed vector
FUNCTION Transp(BYREF V AS LA_V) AS LA_M
  VAR x = LA_M(1, V.Ubound_() + 1) : x.Dat = V.Dat : RETURN x
END FUNCTION

'~------------------------------------------------------------------
'~Gauss-Jordan algorithm for a matrix M(0..r, 0..c) with c >= r
'~from FBmath by jdebord (reviewed)
'~------------------------------------------------------------------
'~On input:
'~  * The submatrix M(0..r, 0..r) contains the system matrix
'~  * The submatrix M(0..r, (r+1)..c) contains the constant vector(s)
'~On output:
'~  * Returns the determinant of the system matrix (0 on error)
'~  * The submatrix M(0..r, 0..r) contains the inverse matrix
'~  * The submatrix M(0..r, (r+1)..c) contains the solution vector(s)
'~------------------------------------------------------------------
FUNCTION GaussJordan OVERLOAD(BYREF M AS LA_M) AS LA_S
  IF M.Ro > M.Co THEN
    LA_Err("incompatible matrix: GaussJordan( M(" & M.Ro & ", " & M.Co & ") )")
    RETURN .0
  END IF
  VAR r = M.Ro - 1, c = M.Co - 1, k = 0
  DIM AS INTEGER prow(r) '~                                line of pivot
  DIM AS INTEGER pcol(r) '~                              column of pivot
  DIM AS LA_S    mcol(r) '~                       a column of the matrix
  DIM AS LA_S    det = 1
  WHILE k <= r
    '~Search for largest pivot in submatrix A[k..r, k..r]
    VAR pvt = LA_G(M)[k * M.Ro + k], ik = k, jk = k
    FOR i AS INTEGER = k TO r
      FOR j AS INTEGER = k TO r
        VAR v = LA_G(M)[j * M.Ro + i]
        IF ABS(v) > ABS(pvt) THEN pvt = v : ik = i : jk = j
      NEXT
    NEXT
    IF ABS(pvt) < LA_Eps THEN
      LA_Err("matrix quasi-singular: GaussJordan( M(" & M.Ro & ", " & M.Co & ") )")
      RETURN .0
    END IF
    prow(k) = ik : pcol(k) = jk '~                   save pivot position
    det *= pvt '~                                     update determinant
    '~          Exchange current row (k) with pivot row (ik), adjust det
    IF ik <> k THEN M.SwapR(k, ik) : det = -det
    '~          Exchange current col (k) with pivot col (ik), adjust det
    IF jk <> k THEN M.SwapC(k, jk) : det = -det
    '~                     Store col. k of M in mcol, set this col. to 0
    FOR i AS INTEGER = 0 TO r
      VAR p = k * M.Ro + i
      IF i <> k THEN
        mcol(i) = LA_G(M)[p]
        LA_G(M)[p] = 0
      ELSE
        mcol(I) = 0
        LA_G(M)[p] = 1
      END IF
    NEXT
    VAR p = k '~                                     transform pivot row
    WHILE p < M.Mx
      LA_G(M)[p] /= pvt
      p += M.Ro
    WEND
    FOR i AS INTEGER = 0 TO r '~                    transform other rows
      IF i <> k THEN
        VAR t = mcol(i), p = 0
        WHILE p < M.Mx
          LA_G(M)[p + i] -= LA_G(M)[p + k] * t
          p += M.Ro
        WEND
      END IF
    NEXT
    k += 1
  WEND
  FOR i AS INTEGER = r TO 0 STEP -1 '~         swap rows of whole matrix
    IF pcol(i) <> i THEN M.SwapR(i, pcol(i))
  NEXT
  FOR j AS INTEGER = r TO 0 STEP -1 '~       swap cols of inverse matrix
    IF prow(j) <> j THEN M.SwapC(j, prow(j))
  NEXT : RETURN det
END FUNCTION

'~Creates the inverse of a matrix
FUNCTION Inv(BYREF M AS LA_M) AS LA_M
  VAR x = M
  IF GaussJordan(x) = .0 THEN x = M
  RETURN x
END FUNCTION


'~*
'~*  Some global matrix operators
'~************************************************************

#MACRO LA_COMP_MS(_OP_)
  FOR i AS INTEGER = 0 TO M.Mx - 1
    IF LA_G(M)[i] _OP_ S THEN RETURN 0
  NEXT : RETURN -1
#ENDMACRO

'~true if all elements of the matrix M are equal to the Scalar
OPERATOR =(BYREF M AS LA_M, BYREF S AS LA_S) AS INTEGER
  LA_COMP_MS(<>)
END OPERATOR

'~true if all elements of the matrix M are unequal to the Scalar
OPERATOR <>(BYREF M AS LA_M, BYREF S AS LA_S) AS INTEGER
  LA_COMP_MS(=)
END OPERATOR

'~true if all elements of the matrix M are smaller than the Scalar
OPERATOR <(BYREF M AS LA_M, BYREF S AS LA_S) AS INTEGER
  LA_COMP_MS(>=)
END OPERATOR

'~true if all elements of the matrix M are greater than the Scalar
OPERATOR >(BYREF M AS LA_M, BYREF S AS LA_S) AS INTEGER
  LA_COMP_MS(<=)
END OPERATOR

'~true if all elements of the matrix M are smaller than or equal to the Scalar
OPERATOR <=(BYREF M AS LA_M, BYREF S AS LA_S) AS INTEGER
  LA_COMP_MS(>)
END OPERATOR

'~true if all elements of the matrix M are greater than or equal to the Scalar
OPERATOR >=(BYREF M AS LA_M, BYREF S AS LA_S) AS INTEGER
  LA_COMP_MS(<)
END OPERATOR


'~true if the Scalar is equal to all elements of the matrix M
OPERATOR =(BYREF S AS LA_S, BYREF M AS LA_M) AS INTEGER
  RETURN M = S
END OPERATOR

'~true if the Scalar is unequal to all elements of the matrix M
OPERATOR <>(BYREF S AS LA_S, BYREF M AS LA_M) AS INTEGER
  RETURN M <> S
END OPERATOR

'~true if the Scalar is smaller than all elements of the matrix M
OPERATOR <(BYREF S AS LA_S, BYREF M AS LA_M) AS INTEGER
  RETURN M > S
END OPERATOR

'~true if the Scalar is greater than all elements of the matrix M
OPERATOR >(BYREF S AS LA_S, BYREF M AS LA_M) AS INTEGER
  RETURN M < S
END OPERATOR

'~true if the Scalar is smaller than or equal to all elements of the matrix M
OPERATOR <=(BYREF S AS LA_S, BYREF M AS LA_M) AS INTEGER
  RETURN M >= S
END OPERATOR

'~true if the Scalar is greater than or equal to all elements of the matrix M
OPERATOR >=(BYREF S AS LA_S, BYREF M AS LA_M) AS INTEGER
  RETURN M <= S
END OPERATOR

'~compare matrixes element-wise, up to the smaller row/col (internal)
#MACRO LA_COMP_MM(_OP_, _T_)
  LA_CHECK_MM(M1, M2, "", " " & #_T_ & " ")
  LA_FOR_MM(M1, M2)
    IF LA_G(M1)[i1] _OP_ LA_G(M2)[i2] THEN RETURN 0
  LA_NEXT_MM()
#ENDMACRO

'~true if matrix M1 = M2
OPERATOR =(BYREF M1 AS LA_M, BYREF M2 AS LA_M) AS INTEGER
  IF M1.Ro <> M2.Ro THEN RETURN 0
  IF M1.Co <> M2.Co THEN RETURN 0
  RETURN M1.Dat = M2.Dat
END OPERATOR

'~true if matrix M1 <> M2
OPERATOR <>(BYREF M1 AS LA_M, BYREF M2 AS LA_M) AS INTEGER
  IF M1.Ro <> M2.Ro THEN RETURN -1
  IF M1.Co <> M2.Co THEN RETURN -1
  RETURN M1.Dat <> M2.Dat
END OPERATOR

'~true if all elements of matrix M1 > M2 equivalents
OPERATOR >(BYREF M1 AS LA_M, BYREF M2 AS LA_M) AS INTEGER
  LA_COMP_MM(<=,>)
END OPERATOR

'~true if all elements of matrix M1 >= M2 equivalents
OPERATOR >=(BYREF M1 AS LA_M, BYREF M2 AS LA_M) AS INTEGER
  LA_COMP_MM(<,>=)
END OPERATOR

'~true if all elements of matrix M1 < M2 equivalents
OPERATOR <(BYREF M1 AS LA_M, BYREF M2 AS LA_M) AS INTEGER
  LA_COMP_MM(>=,<)
END OPERATOR

'~true if all elements of matrix M1 <= M2 equivalents
OPERATOR <=(BYREF M1 AS LA_M, BYREF M2 AS LA_M) AS INTEGER
  LA_COMP_MM(>,<=)
END OPERATOR

'~Matrix = -matrix
OPERATOR -(BYREF M AS LA_M) AS LA_M
  VAR x = M
  FOR i AS INTEGER = 0 TO M.Mx - 1
    LA_G(x)[i] = -LA_G(M)[i]
  NEXT : RETURN x
END OPERATOR


'~Add all Matrix values by Scalar
OPERATOR + (BYREF M AS LA_M, BYVAL S AS LA_S) AS LA_M
  VAR x = M : x += S : RETURN x
END OPERATOR

'~Subtract all Matrix values by Scalar
OPERATOR - (BYREF M AS LA_M, BYVAL S AS LA_S) AS LA_M
  VAR x = M : x -= S : RETURN x
END OPERATOR

'~Multiply all Matrix values by Scalar
OPERATOR * (BYREF M AS LA_M, BYVAL S AS LA_S) AS LA_M
  VAR x = M : x *= S : RETURN x
END OPERATOR

'~Divide all Matrix values by Scalar
OPERATOR / (BYREF M AS LA_M, BYVAL S AS LA_S) AS LA_M
  VAR x = M : IF S THEN x *= 1 / S : RETURN x
  LA_Err("division by zero: M(.,.) / 0.0")
  RETURN x
END OPERATOR

'~Add all Matrix values by Scalar
OPERATOR + (BYVAL S AS LA_S, BYREF M AS LA_M) AS LA_M
  VAR x = M : x += S : RETURN x
END OPERATOR

'~Subtract all Matrix values by Scalar
OPERATOR - (BYVAL S AS LA_S, BYREF M AS LA_M) AS LA_M
  VAR x = -M : x += S : RETURN x
END OPERATOR

'~Multiply all Matrix values by Scalar
OPERATOR * (BYVAL S AS LA_S, BYREF M AS LA_M) AS LA_M
  VAR x = M : x *= S : RETURN x
END OPERATOR

'~Divide a Scalar by a matrix (multiply inverse)
OPERATOR / (BYVAL S AS LA_S, BYREF M AS LA_M) AS LA_M
  RETURN S * Inv(M)
END OPERATOR


'~Matrix = Matrix1 + Matrix2
OPERATOR + (BYREF M1 AS LA_M, BYVAL M2 AS LA_M) AS LA_M
  LA_CHECK_MM(M1, M2, "", " + ")
  VAR rn = IIF(M1.Ro > M2.Ro, M2.Ro, M1.Ro)
  VAR cn = IIF(M1.Co > M2.Co, M2.Co, M1.Co)
  VAR x = LA_M(rn, cn)
  FOR i AS INTEGER = 0 TO x.Mx - 1 '~INDEX!!!
    LA_G(x)[i] = LA_G(M1)[i] + LA_G(M2)[i]
  NEXT : RETURN x
END OPERATOR

'~Matrix = Matrix1 * Matrix2
OPERATOR * (BYREF M1 AS LA_M, BYVAL M2 AS LA_M) AS LA_M
  VAR x = LA_M(M1.Ro, M2.Co)
  IF M1.Co <> M2.Ro THEN
    LA_Err("incompatible matrixes: M(. , " & M1.Co & ") * M(" & M2.Ro & ", .)")
  ELSE
    FOR c AS INTEGER = 0 TO x.Co - 1
      FOR r AS INTEGER = 0 TO x.Ro - 1
        DIM AS LA_S y = .0
        FOR i AS INTEGER = 0 TO M1.Co - 1
          y += LA_G(M1)[i * M1.Ro + r] * LA_G(M2)[c * M2.Ro + i]
        NEXT '~ToDo: Index optimieren -> DO : += : LOOP UNTIL !!!
        LA_G(x)[c * x.Ro + r] = y
      NEXT
    NEXT
  END IF : RETURN x
END OPERATOR

'~Vector = matrix * vector
OPERATOR * (BYREF M AS LA_M, BYVAL V AS LA_V) AS LA_V
  VAR x = LA_V(V.Mx)
  IF M.Co <> x.Mx THEN
    LA_Err("incompatible operands: M(. , " & M.Co & ") * V(" & V.Mx & ")")
  ELSE
    FOR r AS INTEGER = 0 TO x.Mx - 1
      FOR i AS INTEGER = 0 TO M.Co - 1
        LA_G(x)[r] += LA_G(M)[i * M.Ro + r] * LA_G(V)[i]
      NEXT
    NEXT
  END IF : RETURN x
END OPERATOR

'~Divide a matrix by a matrix (left multiply by inverse)
OPERATOR / (BYVAL M1 AS LA_M, BYREF M2 AS LA_M) AS LA_M
  IF M1.Ro = M2.Co THEN RETURN Inv(M2) * M1
  LA_Err("incompatible matrixes: M(" & M1.Ro & ", .) / M(. ," & M2.Co & ")")
  RETURN LA_M(1)
END OPERATOR

'~Divide a Vector by a matrix (left multiply by inverse)
OPERATOR / (BYVAL V AS LA_V, BYREF M AS LA_M) AS LA_V
  IF V.Mx = M.Co THEN RETURN Inv(M) * V
  LA_Err("incompatible operands: V(" & V.Mx & ") / M(. ," & M.Co & ")")
  RETURN LA_V(1)
END OPERATOR
